#!/bin/bash

# In order to use this script create a link as ln -s VDC.sh local/bin/VDC_ENV 
# where VDC_ENV is the code of the environment and place it in your personal local
# bin. You will also need to device where VDC environments live with VDC_HOME

# The expected usage: first issue an VDC_ENV init and then VDC_ENV ansible_command, 
# where VDC_ENV is the code of your environment and the ansible_command is an ansible
# command to be passed into the DVC docker image

if [ $# -eq 0 ]
  then
    echo "No argument supplied: init,ssh,pull..."
    exit 1
fi


VDC_ENV=$(basename $0)

if [ -z ${VDC_HOME} ]; then 
  echo VDC_HOME is undefined
  exit 1
fi

VDC_ENV_HOME=${VDC_HOME}/${VDC_ENV}

if [ ! -d ${VDC_ENV_HOME} ]; then

  echo ${VDC_ENV_HOME} directory not found, creating inital project skeleton

  VDC_BIN=$(realpath $0)
  VDC_SKELETON=$(dirname ${VDC_BIN})/vdc_skeleton

  cp -r ${VDC_SKELETON} $VDC_ENV_HOME
  git init ${VDC_ENV_HOME}

  echo Done.

fi

if [ ! -f ${VDC_ENV_HOME}/.sshkey ]; then
  echo ${VDC_ENV_HOME}/.sshkey not found
  echo Creating the ssh key...
  echo Choose the ${VDC_ENV} password, for ssh and vault.
  ssh-keygen -b 4096 -C vdc@${VDC_ENV} -f ${VDC_ENV_HOME}/.sshkey
fi

if [ -r ${VDC_ENV_HOME}/.environment.sh ]; then
  source ${VDC_ENV_HOME}/.environment.sh
fi

# Use Environment Variables or apply its default

VDC_REPO=${VDC_REPO:-docker.repo}
VDC_VERSION=${VDC_VERSION:-stable}
VDC_IMAGE=${VDC_IMAGE:-$VDC_REPO/vdc:$VDC_VERSION}

VDC_VAULT_TIMEOUT_S=${VDC_VAULT_TIMEOUT_S:-600}
VDC_ALIAS_UTILS=${VDC_ALIAS_UTILS:-ansible ansible-playbook ansible-vault bash}

# Variables not expected to be overidden

VDC_KEY_DESC=${VDC_ENV}_vault
VDC_ARG1U=$(echo $1| tr a-z A-Z)
VDC_TMP_FILE="/tmp/VDC_${VDC_ENV}"
VDC_VAULT_SOCK=${VDC_ENV_HOME}/.vault.sock

# check for init command, will create a new kernel key session and store the vault
# password inside, only this shell can acess it, will also set the expiry on the
# key

if test $VDC_ARG1U = "INIT"; then
  echo Privazio Virtual Datacenter environment ${VDC_ENV}
  cd $VDC_ENV_HOME
  cat << __VAULT_EOF > $VDC_ENV_HOME/.vault
#!/bin/bash
    keyctl pipe \$VDC_KEY_ID
__VAULT_EOF
  chmod 755 $VDC_ENV_HOME/.vault
  cat << __EOF > $VDC_TMP_FILE
    . ~/.bashrc
    PS1="[VDC:$VDC_ENV \W]\$" 

    export VDC_KEY_ID=\$(systemd-ask-password $VDC_ENV password: | keyctl padd user $VDC_KEY_DESC @s)
    keyctl timeout \$VDC_KEY_ID $VDC_VAULT_TIMEOUT_S
    for ALIAS_UTIL in $VDC_ALIAS_UTILS ; do
      alias \$ALIAS_UTIL="${VDC_ENV} \$ALIAS_UTIL"
    done
    export SSH_ASKPASS=$(pwd)/.vault
    ssh-add .sshkey < /dev/null
__EOF
  keyctl session - bash --rcfile $VDC_TMP_FILE -i
  echo "VDC ${VDC_ENV} Session Finished"
  exit 0
fi

#
# Check if pulling a new image is required
#

if test $VDC_ARG1U = "PULL"; then
  docker pull $VDC_IMAGE
  exit 0
fi

#
# Check if we just want to open a console in that environtment
#

if test $VDC_ARG1U = "SSH"; then
  cd $VDC_ENV_HOME 
  shift
  if [ -f $VDC_ENV_HOME/console ]; then
    $VDC_ENV_HOME/console $*
  else

    if [ -f $VDC_ENV_HOME/.ssh.cfg ]; then
      VDC_SSH_CFG="-F $VDC_ENV_HOME/.ssh.cfg"
    fi

    VDC_SSH_FQHN=$(cat inventory | egrep -v "^#" | grep ansible_host | grep $1 | cut -d= -f2)
    shift
    ssh $VDC_SSH_CFG -i $VDC_ENV_HOME/.sshkey -o UserKnownHostsFile=$VDC_ENV_HOME/.ssh/known_hosts $VDC_SSH_FQHN $*
  fi
  exit $?
fi



# From here on, we assumed that calls are from an already Init shell
# Check that the key is present and valid, extend its timeout

if [ -z ${VDC_KEY_ID} ]; then
  echo ${VDC_KEY_ID}
  echo "You need to init session first with:"
  echo ${VDC_ENV} init
  exit 3
else
  keyctl timeout ${VDC_KEY_ID} $VDC_VAULT_TIMEOUT_S || exit 4
fi

# check if the vault named pipe exists

if [ ! -p $VDC_VAULT_SOCK ] ; then
  mkfifo $VDC_VAULT_SOCK
fi

# command execution
VDC_CMD="docker run -it --rm $VDC_DOCKER_OPTS \
  -e ANSIBLE_VAULT_PASSWORD_FILE=/ansible/.vault.sock \
  -e SSH_AUTH_SOCK=$SSH_AUTH_SOCK \
  --user $UID \
  -v ${VDC_ENV_HOME}:/ansible \
  -v ${VDC_ENV_HOME}/.ssh:/home/ansible/.ssh \
  -v $SSH_AUTH_SOCK:$SSH_AUTH_SOCK \
  ${VDC_IMAGE}"


#this does not seem to do the job and setenfore 0 is still required
#chcon -Rt svirt_sandbox_file_t $VDC_ENV_HOME

# Allow the utility to read the vault password (once)
keyctl pipe $VDC_KEY_ID > $VDC_ENV_HOME/.vault.sock &
TMP_VAULT_PID=$!

#echo ${VDC_CMD} $*
${VDC_CMD} $*

# kill the vault socker if still running
ps -p $TMP_VAULT_PID > /dev/null && kill $TMP_VAULT_PID 
