import unittest

from plugins.filter.pvzca import sign, is_valid_certificate, verify
from OpenSSL import crypto
import os
from datetime import datetime,timedelta

del os.environ['LANG']

csr = '''-----BEGIN CERTIFICATE REQUEST-----
MIIE4zCCAssCAQAwYzELMAkGA1UEBhMCU0UxFTATBgNVBAoMDFZhbGxldGVjaCBB
QjELMAkGA1UECwwCSVQxDTALBgNVBAMMBHB2ejExITAfBgkqhkiG9w0BCQEWEmhl
bGxvQHZhbGxldGVjaC5ldTCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIB
AMkZp8RGwzQnOOyhjQkcCIvEVjtze+uQJQJK0gXaMweR6F5tbzwYt3CihzzqMbSN
iYf1PQqqFWcPvkvLBKyGDNojX128X4D5c79UCSobCzxyYLk0h48VKOlxi7DlPS08
10xixiKUkmUNAzvrVWMjN3vQEkk5oa2s+kWsrS1N4F5q7ZdlFsAmJl9JWxATNaim
PxDKnYiIQGL64WtuEBJ7ABePH8tvj4+NvcBDWuXgzjyVYpc4SbrYv+PRpwMlOaId
jVuckaDjbtGUwXnUw1agld6lq6veoyFcsU2r7FDFSccHoRMGmmPsKC60PTIUv048
3e72R8DSq8G90yPFDxjQd0qPw6P86OoJKl2vvBAjW9DdF5UgNdrPJ9+i64pWH/WE
XhIjLQ0ml94b3JwpM+jJNx2bLS2KXZ2QPPprKcULGLk/HGBrk1J5NkZCeVtYoTsW
tqmeMlCuxBn86HvRbNHqKjr/zJVVE09l5C2T4qG8Yedk8IIiT/iapRDEg9kIMV+y
nhyJFYp21JbgsLiP/SSyFZcEGMsWxZpMKezlQIxsYyPHThTcRJuMQelpKuYeXLVA
UWkPi+WDvOc9IDk2JiVk7nv3/ZrdB06nzn0dUXEy+np0cS0mL900iJ/Xuw0Wgsh2
YoNB2zwoI+MGgHa5NgL3kYuilrztWRHOK4CZINjvxZBPAgMBAAGgOzA5BgkqhkiG
9w0BCQ4xLDAqMCgGA1UdEQQhMB+CBHB2ejGCC2plbmtpbnMudnRrggpuZXh1czMu
dnRrMA0GCSqGSIb3DQEBCwUAA4ICAQALwux0JNn0sBQSDZot+W+2WCFAIaDUJtPj
iblWPPr8gP/7LgLHMJtlisBW1x6WAayfM6lb8kHpQ6HFxyzXfL8nA33gRlt9jGl4
S71HatigCASI5VWhxKXt0MP3AOWSgUAQ0CJ4xwUK1si0xI3PLW2HR8zful1rfkUj
s+aYV24zPnlQAyYaDKAM3XWO6IAB4kQuYOWSqQ1rdx/IeILNQrf/Y4DdwQ+TBJ5d
oJg298YA9epQ+j77vKNTqmfsxMrrwtfFjP8V9+WUp7Jl7osQIMv+gulFW01BVp7Q
U2IAkYOP0mwwdvPj9uT4xLh7tiLCGnKtW41j47sgdPndJ5UeJPlcZK6DAlnhMbfn
r2SjbFkSR5veQElaPypNpaIZAqltu6mIQzmf+feTeKkIvk6E1ej5mjqe87OXnmKj
jhu1WhF3X8Nh7xYvBjh/C+qMzxhqy7Aq77+IXuek4Rr03ES0+7CPSo3+0Vil1XPy
yO2W3/ucpM8VsQAFLaRZo8+ysMY0AMdr6vQ47Z2pSMPEb5jIC3dloCEmTlhhsykx
K6Wt2V6tKkpGsQzxKNFT4bRFXmckgDFQ2cXkEffQXokNSz7n5yR9YHciycCnpoxL
jcWvEPNmXURQufQ5l9OesMZg2MjO2Ci8q+o7am8jhnAhZnfIQaGsjvCMycjWa+cu
Hr9XZwRi4w==
-----END CERTIFICATE REQUEST-----
'''

ca = '''-----BEGIN CERTIFICATE-----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-----END CERTIFICATE-----'''

key = '''-----BEGIN RSA PRIVATE KEY-----
Proc-Type: 4,ENCRYPTED
DEK-Info: DES-EDE3-CBC,738B098A115C5206
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-----END RSA PRIVATE KEY-----'''

keypass = u"123456"

class PVZCaTest(unittest.TestCase):
    def test_can_read_ca_cert(self):
        caCert = crypto.load_certificate(crypto.FILETYPE_PEM, ca)
        self.assertEqual(caCert.get_serial_number(), 1)

    def test_can_read_cert_req(self):
        certReq = crypto.load_certificate_request(crypto.FILETYPE_PEM, csr)
        self.assertTrue(certReq.verify(certReq.get_pubkey()))

    def test_can_read_password_protected_key(self):
        caKey = crypto.load_privatekey(crypto.FILETYPE_PEM, key, keypass.encode())
        self.assertEqual(caKey.bits(), 4096)

    def test_read_invalid_pem(self):
        with self.assertRaises(Exception):
            try:
                caKey = crypto.load_privatekey(crypto.FILETYPE_PEM, '\n', keypass)
            except Exception as e:
                raise Exception("Could not read key: %s " % e)

    def test_can_sign(self):
        caCert = crypto.load_certificate(crypto.FILETYPE_PEM, ca)
        certReq = crypto.load_certificate_request(crypto.FILETYPE_PEM, csr)
        caKey = crypto.load_privatekey(crypto.FILETYPE_PEM, key, keypass.encode())
        digest = "sha256"
        notBefore = 0
        notAfter = 30*24*60*60
        cert = crypto.X509()
        cert.set_serial_number(2)
        cert.gmtime_adj_notBefore(notBefore)
        cert.gmtime_adj_notAfter(notAfter)
        cert.set_issuer(caCert.get_subject())
        cert.set_subject(certReq.get_subject())
        cert.set_pubkey(certReq.get_pubkey())
        cert.add_extensions(certReq.get_extensions())
        cert.sign(caKey, digest)
        ret = crypto.dump_certificate(crypto.FILETYPE_PEM, cert)
        self.assertTrue(is_valid_certificate(cert, certReq, 10))
        self.assertEqual(cert.get_issuer(), caCert.get_subject())

    def test_can_sign_with_module(self):
        ret = sign(csr, ca, key, keypass=keypass)
        ret

    def test_can_verify_signed(self):
        cert_txt = sign(csr, ca, key, keypass=keypass)
        ca_cert = crypto.load_certificate(crypto.FILETYPE_PEM, ca)
        cert = crypto.load_certificate(crypto.FILETYPE_PEM, cert_txt)
        store = crypto.X509Store()
        store.add_cert(ca_cert)
        store_ctx = crypto.X509StoreContext(store, cert)
        self.assertEqual(store_ctx.verify_certificate(), None)

    def test_can_verify_signed_with_module(self):
        cert_txt = sign(csr, ca, key, keypass=keypass)
        ca_cert = crypto.load_certificate(crypto.FILETYPE_PEM, ca)
        cert = crypto.load_certificate(crypto.FILETYPE_PEM, cert_txt)
        self.assertTrue(verify(cert, ca_cert))


